-- | Provides security for pure computations (trusted).
module Sec where

import Lattice

-- Sec

-- | This type represents values of type a at confidentility level s. 
newtype Sec s a = MkSec a


instance Functor (Sec s) where
  h `fmap` (MkSec x) = MkSec (h x)


instance Monad (Sec s) where
  return x = sec x

  MkSec a >>= k =
    MkSec (let MkSec b = k a in b)

sec :: a -> Sec s a
sec x = MkSec x

{-| Open the abstraction provided by 'Sec'. 
    However, it is only possible to obtain a when  providing the key s.
-}
open :: Sec s a -> s -> a
open (MkSec a) s = s `seq` a

-- | Lift information into higher positions on the security lattice.
up :: Less s s' => Sec s a -> Sec s' a
up sec_s@(MkSec a) = less s s' `seq` sec_s' 
                     where (sec_s') = MkSec a 
                           s      = unSecType sec_s 
                           s'     = unSecType sec_s'

fromIntegrity :: Sec (s,i) a -> Sec s a
fromIntegrity sec_s@(MkSec a) = MkSec a

-- | Break the abstraction provide by 'Sec'. It is used only in trusted code!
reveal :: Sec s a -> a
reveal (MkSec a) = a

-- | Internal function, not exported. For type-checking purposes.
unSecType :: Sec s a -> s 
unSecType _ = undefined


-- | Reveal a when its confidentiality level is public.
public :: Sec s a -> a 
public (MkSec a) = a
